@@ -2,6 +2,9 @@ import logging
 import time
 
 import typer
+from click import BOOL
+import keyring
+import getpass
 
 from fluxvault import FluxAgent, FluxKeeper
 
@@ -10,6 +13,18 @@ PREFIX = "FLUXVAULT"
 app = typer.Typer(rich_markup_mode="rich", add_completion=False)
 
 
+class colours:
+    HEADER = "\033[95m"
+    OKBLUE = "\033[94m"
+    OKCYAN = "\033[96m"
+    OKGREEN = "\033[92m"
+    WARNING = "\033[93m"
+    FAIL = "\033[91m"
+    ENDC = "\033[0m"
+    BOLD = "\033[1m"
+    UNDERLINE = "\033[4m"
+
+
 def configure_logs(log_to_file, logfile_path, debug):
     vault_log = logging.getLogger("fluxvault")
     aiotinyrpc_log = logging.getLogger("aiotinyrpc")
@@ -34,6 +49,37 @@ def configure_logs(log_to_file, logfile_path, debug):
         vault_log.addHandler(file_handler)
 
 
+def yes_or_no(question, default="yes"):
+    """Ask a yes/no question via raw_input() and return their answer.
+
+    "question" is a string that is presented to the user.
+    "default" is the presumed answer if the user just hits <Enter>.
+            It must be "yes" (the default), "no" or None (meaning
+            an answer is required of the user).
+
+    The "answer" return value is True for "yes" or False for "no".
+    """
+    valid = {"yes": True, "y": True, "ye": True, "no": False, "n": False}
+    if default is None:
+        prompt = " [yes/no] "
+    elif default == "yes":
+        prompt = f" [{colours.OKGREEN}Yes{colours.ENDC}] "
+    elif default == "no":
+        prompt = f" [{colours.OKGREEN}No{colours.ENDC}] "
+    else:
+        raise ValueError("invalid default answer: '%s'" % default)
+
+    while True:
+        print(question + prompt, end="")
+        choice = input().lower()
+        if default is not None and choice == "":
+            return valid[default]
+        elif choice in valid:
+            return valid[choice]
+        else:
+            print("Please respond with 'yes' or 'no' " "(or 'y' or 'n').\n")
+
+
 @app.command()
 def keeper(
     vault_dir: str = typer.Option(
@@ -76,22 +122,49 @@ def keeper(
         "",
         envvar=f"{PREFIX}_AGENT_IPS",
         show_envvar=False,
+        help="If your not using app name to determine ips",
     ),
-    signing_key: str = typer.Option(
-        "",
-        envvar=f"{PREFIX}_SIGNING_KEY",
+    sign_connections: bool = typer.Option(
+        False,
+        "--sign-connections",
+        "-q",
+        envvar=f"{PREFIX}_SIGN_CONNECTIONS",
         show_envvar=False,
+        help="Whether or not to sign outbound connections",
+    ),
+    zelid: str = typer.Option(
+        "",
+        envvar=f"{PREFIX}_ZELID",
+        show_envvar=False,
+        help="This is used to associate private key in keychain",
     ),
 ):
 
     agent_ips = agent_ips.split(",")
     agent_ips = list(filter(None, agent_ips))
 
+    if sign_connections:
+        if not zelid:
+            raise ValueError("zelid must be provided if signing connections (keyring)")
+
+        signing_key = keyring.get_password("fluxvault_app", zelid)
+
+        if not signing_key:
+            signing_key = getpass.getpass(
+                f"\n{colours.OKGREEN}** WARNING **\n\nYou are about to enter your private key into a 3rd party application. Please make sure your are comfortable doing so. If you would like to review the code to make sure your key is safe... please visit https://github.com/RunOnFlux/FluxVault to validate the code.{colours.ENDC}\n\n Please enter your private key (in WIF format):\n"
+            )
+            store_key = yes_or_no(
+                "Would you like to store your private key in your device's secure store?\n\n(macOS: keyring, Windows: Windows Credential Locker, Ubuntu: GNOME keyring.\n\n This means you won't need to enter your private key every time this program is run.",
+            )
+            if store_key:
+                keyring.set_password("fluxvault_app", zelid, signing_key)
+
     flux_keeper = FluxKeeper(
         vault_dir=vault_dir,
         comms_port=comms_port,
         app_name=app_name,
         agent_ips=agent_ips,
+        sign_connections=sign_connections,
         signing_key=signing_key,
     )
 
@@ -177,6 +250,12 @@ def agent(
         show_envvar=False,
         help="Expects all keeper connections to be signed",
     ),
+    zelid: str = typer.Option(
+        "",
+        envvar=f"{PREFIX}_ZELID",
+        show_envvar=False,
+        help="Testing only... if you aren't running this on a Fluxnode",
+    ),
 ):
 
     whitelisted_addresses = whitelisted_addresses.split(",")
@@ -194,6 +273,7 @@ def agent(
         whitelisted_addresses=whitelisted_addresses,
         verify_source_address=verify_source_address,
         signed_vault_connections=signed_vault_connections,
+        zelid=zelid,
     )
 
     agent.run()
