@@ -36,6 +36,7 @@ class FluxKeeper:
         app_name: str = "",
         agent_ips: list = [],
         extensions: FluxVaultExtensions = FluxVaultExtensions(),
+        sign_connections: bool = False,
         signing_key: str = "",
     ):
         self.app_name = app_name
@@ -47,8 +48,11 @@ class FluxKeeper:
         self.protocol = JSONRPCProtocol()
         self.vault_dir = vault_dir
 
+        if not signing_key and sign_connections:
+            raise ValueError("Signing key must be provided if signing connections")
+
         auth_provider = None
-        if signing_key:
+        if signing_key and sign_connections:
             auth_provider = SignatureAuthProvider(key=signing_key)
 
         for ip in self.agent_ips:
